unit mainform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ActnList, ComCtrls,
  StdCtrls, ExtCtrls, Buttons;

type

  { TMain_Form }

  TMain_Form = class(TForm)
    StartHelp_Button: TButton;
    HelpExec_Button: TButton;
    HelpEdit_Button: TButton;
    Lazarus_Label: TLabel;
    VarInputOK_Button: TButton;
    VarInput_Memo: TMemo;
    VarInputTitle_Label: TLabel;
    Run_Button: TBitBtn;
    StepText_Memo: TMemo;
    StepVariableName_Edit: TEdit;
    StepDelay_Label: TLabel;
    StepTitle_Edit: TEdit;
    Label10: TLabel;
    Label11: TLabel;
    StepType_Label: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    StepEdit_Button: TBitBtn;
    Label7: TLabel;
    SaveAs_Button: TButton;
    Save_Button: TButton;
    Data_GroupBox: TGroupBox;
    FileName_Edit: TEdit;
    Label3: TLabel;
    Hint_Label: TLabel;
    Label6: TLabel;
    StepDown_Button: TBitBtn;
    StepUp_Button: TBitBtn;
    StepDel_Button: TBitBtn;
    StepAdd_Button: TBitBtn;
    Edit_Button: TButton;
    EditSteps_ListBox: TListBox;
    Next_Button: TButton;
    ExecSteps_ListBox: TListBox;
    Delay_Label: TLabel;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Delay_Timer: TTimer;
    InputData_TabSheet: TTabSheet;
    Title_Label: TLabel;
    Text_Memo: TMemo;
    StartStop_Button: TButton;
    Label5: TLabel;
    Open_Button: TButton;
    Create_Button: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Site_Label: TLabel;
    Label4: TLabel;
    PageControl: TPageControl;
    Start_TabSheet: TTabSheet;
    EditTask_TabSheet: TTabSheet;
    Task_TabSheet: TTabSheet;
    procedure Create_ButtonClick(Sender: TObject);
    procedure EditSteps_ListBoxClick(Sender: TObject);
    procedure Edit_ButtonClick(Sender: TObject);
    procedure HelpEdit_ButtonClick(Sender: TObject);
    procedure HelpExec_ButtonClick(Sender: TObject);
    procedure Lazarus_LabelClick(Sender: TObject);
    procedure Next_ButtonClick(Sender: TObject);
    procedure OnShow(Sender: TObject);
    procedure OnTimer(Sender: TObject);
    procedure Open_ButtonClick(Sender: TObject);
    procedure Run_ButtonClick(Sender: TObject);
    procedure SaveAs_ButtonClick(Sender: TObject);
    procedure Save_ButtonClick(Sender: TObject);
    procedure Site_LabelClick(Sender: TObject);
    procedure StartHelp_ButtonClick(Sender: TObject);
    procedure StartStop_ButtonClick(Sender: TObject);
    procedure StepAdd_ButtonClick(Sender: TObject);
    procedure StepDblClick(Sender: TObject);
    procedure StepDel_ButtonClick(Sender: TObject);
    procedure StepDown_ButtonClick(Sender: TObject);
    procedure StepEdit_ButtonClick(Sender: TObject);
    procedure StepUp_ButtonClick(Sender: TObject);
    procedure VarInputOK_ButtonClick(Sender: TObject);
  private
    procedure processFileOpen(FileName: string);
    procedure fillSteps;
    procedure enableControls;
    procedure showStep(Index: integer);
    function textValue(stepIndex: integer): string;
    procedure prepareExec;
    procedure setStep(Index: integer);
    function variableIndex(varName: string): integer;
  public

  end;

var
  Main_Form: TMain_Form;

implementation

{$R *.lfm}

uses lclintf, taskfile, stepeditform, stringfuncs, Clipbrd, lcltype, helpform;

var
  Task: TTaskFile;
  CurrentStep: integer;
  Started: boolean;
  Variables: array of TVariable;

{ TMain_Form }

procedure TMain_Form.processFileOpen(FileName: string);
begin
  Task.FileName := FileName;
  if not Task.Open then
    ShowMessage(Task.Error)
  else
  begin
    FileName_Edit.Text := FileName;
    prepareExec;
    PageControl.ActivePage := Task_TabSheet;
  end;
end;

procedure TMain_Form.fillSteps;
var
  i: integer;
begin
  EditSteps_ListBox.Items.Clear;
  ExecSteps_ListBox.Items.Clear;
  if Length(Task.Steps) > 0 then
    for i := 0 to Length(Task.Steps) - 1 do
    begin
      EditSteps_ListBox.Items.Add(Task.Steps[i].Title);
      ExecSteps_ListBox.Items.Add(Task.Steps[i].Title);
    end;
  EditSteps_ListBox.ItemIndex := -1;
  ExecSteps_ListBox.ItemIndex := -1;
  StepUp_Button.Enabled := False;
  StepDown_Button.Enabled := False;
  StepDel_Button.Enabled := False;
  Hint_Label.Visible := True;
  Data_Groupbox.Visible := False;
end;

procedure TMain_Form.Site_LabelClick(Sender: TObject);
begin
  OpenURL(Site_Label.Caption);
end;

procedure TMain_Form.StartHelp_ButtonClick(Sender: TObject);
begin
  Help_Form.displayHelp(1);
end;

procedure TMain_Form.StartStop_ButtonClick(Sender: TObject);
begin
  if not Started then
  begin
    StartStop_Button.Caption := 'Стоп';
    Edit_Button.Enabled := False;
    Started := True;
    SetLength(Variables, 0);
    setStep(0);
  end
  else
  begin
    Delay_Timer.Enabled := False;
    Started := False;
    StartStop_Button.Caption := 'Старт';
    Edit_Button.Enabled := True;
  end;
end;

procedure TMain_Form.StepAdd_ButtonClick(Sender: TObject);
var
  i: integer;
begin
  StepEdit_Form.NewRec := True;
  if StepEdit_Form.ShowModal = mrOk then
  begin
    i := Task.addStep(StepEdit_Form.getData);
    fillSteps;
    EditSteps_ListBox.ItemIndex := i;
    enableControls;
    showStep(i);
  end;
end;

procedure TMain_Form.StepDblClick(Sender: TObject);
begin
  if (ExecSteps_Listbox.ItemIndex <> -1) and (ExecSteps_Listbox.ItemIndex <
    ExecSteps_Listbox.Items.Count) then
  begin
    if not Started then
    begin
      StartStop_ButtonClick(Sender);
      Delay_Label.Caption := '';
      Title_Label.Caption := '';
      Text_Memo.Lines.Clear;
      Next_Button.Enabled := False;
    end;
    setStep(ExecSteps_Listbox.ItemIndex);
  end;
end;

procedure TMain_Form.StepDel_ButtonClick(Sender: TObject);
var
  i: integer;
begin
  if MessageBox(Handle, 'Вы уверены?', 'Удалить шаг',
    MB_YESNO) = idYes then
  begin
    i := Task.delStep(EditSteps_Listbox.ItemIndex);
    FillSteps;
    EditSteps_Listbox.ItemIndex := i;
    showStep(i);
  end;

end;

procedure TMain_Form.StepDown_ButtonClick(Sender: TObject);
var
  i: integer;
begin
  i := Task.moveDown(EditSteps_Listbox.ItemIndex);
  FillSteps;
  EditSteps_Listbox.ItemIndex := i;
  showStep(i);
end;

procedure TMain_Form.StepEdit_ButtonClick(Sender: TObject);
begin
  StepEdit_Form.setData(Task.Steps[EditSteps_ListBox.ItemIndex]);
  StepEdit_Form.NewRec := False;
  if StepEdit_Form.ShowModal = mrOk then
  begin
    Task.Steps[EditSteps_ListBox.ItemIndex] := StepEdit_Form.getData;
    EditSteps_ListBox.Items.Strings[EditSteps_ListBox.ItemIndex] :=
      Task.Steps[EditSteps_ListBox.ItemIndex].Title;
    ShowStep(EditSteps_ListBox.ItemIndex);
  end;
end;

procedure TMain_Form.StepUp_ButtonClick(Sender: TObject);
var
  i: integer;
begin
  i := Task.moveUp(EditSteps_Listbox.ItemIndex);
  FillSteps;
  EditSteps_Listbox.ItemIndex := i;
  showStep(i);
end;

procedure TMain_Form.VarInputOK_ButtonClick(Sender: TObject);
var
  i: integer;
begin
  PageControl.ActivePage := Task_TabSheet;
  i := variableIndex(Task.Steps[CurrentStep].VariableName);
  Variables[i].Value := VarInput_Memo.Text;
  if (CurrentStep < Length(Task.Steps) - 1) then
    Next_ButtonClick(self);
end;

procedure TMain_Form.Create_ButtonClick(Sender: TObject);
begin
  Task.Init;
  Caption := 'Новый - VL:Копипаст';
  FileName_Edit.Text := '';
  FillSteps;
  PageControl.ActivePage := EditTask_TabSheet;
end;

procedure TMain_Form.enableControls;
begin
  StepUp_Button.Enabled := (EditSteps_ListBox.ItemIndex > 0);
  StepDown_Button.Enabled := (EditSteps_Listbox.ItemIndex <
    EditSteps_Listbox.Items.Count - 1) and (EditSteps_ListBox.ItemIndex <> -1);
  StepDel_Button.Enabled := (EditSteps_ListBox.ItemIndex <> -1);
  Hint_Label.Visible := not StepDel_Button.Enabled;
  Data_Groupbox.Visible := StepDel_Button.Enabled;
  StepEdit_Button.Enabled := StepDel_Button.Enabled;
end;

procedure TMain_Form.EditSteps_ListBoxClick(Sender: TObject);
begin
  EnableControls;
  ShowStep(EditSteps_Listbox.ItemIndex);
end;

procedure TMain_Form.Edit_ButtonClick(Sender: TObject);
begin
  PageControl.ActivePage := EditTask_Tabsheet;
end;

procedure TMain_Form.HelpEdit_ButtonClick(Sender: TObject);
begin
  Help_Form.displayHelp(2);
end;

procedure TMain_Form.HelpExec_ButtonClick(Sender: TObject);
begin
  Help_Form.displayHelp(5);
end;

procedure TMain_Form.Lazarus_LabelClick(Sender: TObject);
begin
  OpenURL('https://www.lazarus-ide.org/');
end;

procedure TMain_Form.Next_ButtonClick(Sender: TObject);
begin
  setStep(CurrentStep + 1);
end;

procedure TMain_Form.OnShow(Sender: TObject);
begin
  if ParamCount > 0 then
    processFileOpen(ParamStr(1))
    else PageControl.ActivePage := Start_Tabsheet;
end;

procedure TMain_Form.OnTimer(Sender: TObject);
begin
  Next_ButtonClick(Sender);
end;

procedure TMain_Form.Open_ButtonClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    processFileOpen(OpenDialog.FileName);
end;

procedure TMain_Form.Run_ButtonClick(Sender: TObject);
begin
  PageControl.ActivePage := Task_TabSheet;
  prepareExec;
end;

procedure TMain_Form.SaveAs_ButtonClick(Sender: TObject);
begin
  if SaveDialog.Execute then
  begin
    Task.FileName := SaveDialog.FileName;
    FileName_Edit.Text := SaveDialog.FileName;
    if not Task.Save then
      ShowMessage(Task.Error)
    else
      Caption := ExtractFileName(Task.FileName) + ' - VL:Копипаст';
  end;
end;

procedure TMain_Form.Save_ButtonClick(Sender: TObject);
begin
  if FileName_Edit.Text = '' then
    SaveAs_ButtonClick(Sender)
  else begin
   Task.FileName:=FileName_Edit.Text;
   if not Task.Save then
    ShowMessage(Task.Error);
  end;
end;

procedure TMain_Form.showStep(Index: integer);
begin
  if Index = -1 then
  begin
    StepType_Label.Caption := '';
    StepTitle_Edit.Text := '';
    StepDelay_Label.Caption := '';
    StepVariableName_Edit.Text := '';
    StepText_Memo.Text := '';
    Data_Groupbox.Visible := False;
    Hint_Label.Visible := True;
  end
  else
  begin
    StepType_Label.Caption := StepTypes[Task.Steps[Index].StepType];
    StepTitle_Edit.Text := Task.Steps[Index].Title;
    StepDelay_Label.Caption := IntToStr(Task.Steps[Index].Delay);
    StepVariableName_Edit.Text := Task.Steps[Index].VariableName;
    StepText_Memo.Text := Task.Steps[Index].TextToCopy;
    Data_Groupbox.Visible := True;
    Hint_Label.Visible := False;
  end;
  enableControls;
end;

function TMain_Form.textValue(stepIndex: integer): string;
var
  i, j: integer;
begin
  if stepIndex > Length(Task.Steps) - 1 then
  begin
    Result := '';
    Exit;
  end;
  Result := Task.Steps[stepIndex].TextToCopy;
  if Length(Variables) > 0 then
    for i := 0 to Length(Variables) - 1 do
      Result := str_replace(Result, Variables[i].Name, Variables[i].Value, j);
end;

procedure TMain_Form.setStep(Index: integer);
var
  s: string;
begin
  Delay_Timer.Enabled := False;
  if Index = Length(Task.Steps) - 1 then
  begin
    Next_Button.Enabled := False;
    ShowMessage('Это последний шаг.');
    Started := False;
    StartStop_Button.Caption := 'Старт';
    Edit_Button.Enabled := True;
  end
  else
  begin
    Next_Button.Enabled := True;
    Edit_Button.Enabled := False;
    Started := True;
  end;
  CurrentStep := Index;
  ExecSteps_Listbox.ItemIndex := CurrentStep;
  if (Task.Steps[Index].StepType = 0) or (Task.Steps[Index].StepType = 1) then
  begin
    Title_Label.Caption := Task.Steps[Index].Title;
    Text_Memo.Text := textValue(Index);
    if (Task.Steps[Index].StepType = 0) then
    begin
      s := Text_Memo.Text;
      Clipboard.SetTextBuf(PChar(s));
    end
    else
    begin
      Clipboard.Clear;
    end;
    if (Task.Steps[Index].Delay <> 0) then
      Delay_Label.Caption := 'След. шаг через ' +
        IntToStr(Task.Steps[Index].Delay) + ' с.'
    else
      Delay_Label.Caption := '"Далее" для продолжения';
    if (Task.Steps[Index].Delay <> 0) and (Index < Length(Task.Steps) - 1) then
    begin
      Delay_Timer.Interval := Task.Steps[Index].Delay * 1000;
      Delay_Timer.Enabled := True;
    end;
  end;
  if Task.Steps[Index].StepType = 2 then
  begin
    // ввод данных
    VarInputTitle_Label.Caption := Task.Steps[Index].Title;
    VarInput_Memo.Lines.Clear;
    PageControl.ActivePage := InputData_TabSheet;
  end;
end;

procedure TMain_Form.prepareExec;
begin
  fillSteps;
  StartStop_Button.Enabled := (ExecSteps_Listbox.Items.Count > 0);
  StartStop_Button.Caption := 'Старт';
  Delay_Label.Caption := '';
  if StartStop_Button.Enabled then
    Title_Label.Caption := 'Нажмите Старт'
  else
    Title_Label.Caption :=
      'Сценарий пуст, выполнение невозможно';
  Text_Memo.Lines.Clear;
  Next_Button.Enabled := False;
  Started := False;
  if StartStop_Button.Enabled then
  begin
    ExecSteps_Listbox.ItemIndex := 0;
  end;
end;

function TMain_Form.variableIndex(varName: string): integer;
var
  i: integer;
begin
  Result := -1;
  if Length(Variables) > 0 then
    for i := 0 to Length(Variables) - 1 do
      if Variables[i].Name = varName then
      begin
        Result := i;
        Break;
      end;
  if Result = -1 then
  begin
    Result := Length(Variables);
    SetLength(Variables, Result + 1);
    Variables[Result].Name := varName;
    Variables[Result].Value := '';
  end;
end;

end.
